--COMMENTS FOR EACH TABLE: 0 = NOT, 1 = YES--
--FIELDS        //FIELDS SET UP
--PRIMARY KEYS        //PRIMARY KEYS SET UP
--NULLS            //NULLABLE FIELDS SELECTED
--UNIQUES        //UNIQUE FIELDS DETERMINED
--FOREIGN KEYS        //FOREIGN KEYS DELEGATED
--CASCADES        //ON DELETE, ON UPDATE COMMANDS SET

-- THE PARSING MARKER FOR SQLITE_CONNECTOR:  DOUBLE#

------TABLES

--TODO_PROJECT_ALARM_RECURRENCE
--FIELDS 1 ; PRIMARY KEYS 1 ;NULLS 1 ;UNIQUES 1; FOREIGN KEYS 1; CASCADES 1; 
DROP TABLE IF EXISTS "TODO_PROJECT";--##
CREATE TABLE "TODO_PROJECT" (
    "todo_id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,--Milestone v0.1
    "recurrence_id" INTEGER NOT NULL ,
    "start_date" DATETIME DEFAULT CURRENT_DATE,
    "end_date" DATETIME DEFAULT CURRENT_DATE,
    "time_zone" TEXT DEFAULT "+00:00",
    "last_mod_date" DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "creation_date" DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
    "status" INTEGER NOT NULL DEFAULT 0,
    "is_next" BOOL NOT NULL DEFAULT "false",
    "is_done" BOOL NOT NULL DEFAULT "false",
    "is_sample" BOOL NOT NULL DEFAULT "false",
    "note" TEXT DEFAULT "Something to do",
    "format_string" TEXT ,
    "is_deleted" BOOL NOT NULL DEFAULT "false"
    );--##

--FIELDS 1 ; PRIMARY KEYS 1 ;NULLS 1 ;UNIQUES 1; FOREIGN KEYS 1; CASCADES 1; 
DROP TABLE IF EXISTS "ALARM";--##
CREATE TABLE "ALARM" (
    "alarm_id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
    "todo_id" INTEGER NOT NULL DEFAULT 0,
    "last_mod_date" DATETIME NOT NULL  DEFAULT CURRENT_TIMESTAMP,
    "alarm_time" DATETIME NOT NULL DEFAULT 0,
    "is_active" BOOL NOT NULL  DEFAULT 0,
    "snooze" INT DEFAULT 0,
    "snooze_interval" INTEGER DEFAULT 5,
    "is_deleted" BOOL NOT NULL DEFAULT "false",
    FOREIGN KEY("todo_id") REFERENCES "TODO_PROJECT"("todo_id") ON DELETE CASCADE ON UPDATE CASCADE
    );--##

--FIELDS 1 ; PRIMARY KEYS 1 ;NULLS 1 ;UNIQUES 1; FOREIGN KEYS 1; CASCADES 1; 
DROP TABLE IF EXISTS "RECURRENCE";--##
CREATE TABLE "RECURRENCE" (
    "recurrence_id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
    "sample_id" INTEGER NOT NULL,
    "last_mod_date" DATETIME NOT NULL  DEFAULT CURRENT_TIMESTAMP,
    "is_active" BOOL NOT NULL  DEFAULT  0,
    "repeats" INTEGER DEFAULT 0,
    "repeats_every" INTEGER DEFAULT 0,
    "repeats_on" TEXT,
    "range_start" DATETIME,
    "range_end" DATETIME,
    "instancing_date" DATETIME NOT NULL  DEFAULT "2010-06-01 00:00:00.000",
    "is_deleted" BOOL NOT NULL DEFAULT "false"
    );--##

--FIELD_FIELD_TYPE
--FIELDS 1 ; PRIMARY KEYS 1 ;NULLS 1 ;UNIQUES 1; FOREIGN KEYS 1; CASCADES 1; 
DROP TABLE IF EXISTS "FIELD_TYPE";--##
CREATE TABLE "FIELD_TYPE" (
    "field_type_id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,
    "last_mod_date" DATETIME NOT NULL  DEFAULT CURRENT_TIMESTAMP,
    "name" TEXT NOT NULL DEFAULT "Default field type name",
    "is_deleted" BOOL NOT NULL DEFAULT "false",
    UNIQUE ("name")
    );--##

--FIELDS 1 ; PRIMARY KEYS 1 ;NULLS 1 ;UNIQUES 1; FOREIGN KEYS 1; CASCADES 1; 
DROP TABLE IF EXISTS "FIELD";--##
CREATE TABLE "FIELD" (
    "field_id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ,
    "todo_id" INTEGER NOT NULL DEFAULT 0,
    "field_type_id" INTEGER NOT NULL ,
    "last_mod_date" DATETIME NOT NULL  DEFAULT CURRENT_TIMESTAMP,
    "value" TEXT NOT NULL  DEFAULT "Default field value",
    "db_id" TEXT,
    "contact_id" TEXT,
    "is_deleted" BOOL NOT NULL DEFAULT "false",
    FOREIGN KEY("todo_id") REFERENCES "TODO_PROJECT"("todo_id") ON DELETE CASCADE ON UPDATE CASCADE,
    FOREIGN KEY("field_type_id") REFERENCES "FIELD_TYPE"("field_type_id") ON DELETE CASCADE ON UPDATE CASCADE
    );--##

--BINARY
--FIELDS 1 ; PRIMARY KEYS 1 ;NULLS 1 ;UNIQUES 1; FOREIGN KEYS 1; CASCADES 1; 
DROP TABLE IF EXISTS "BINARY";--##
CREATE TABLE "BINARY" (
    "binary_id" INTEGER PRIMARY KEY NOT NULL ,
    "todo_id" INTEGER NOT NULL DEFAULT 0,
    "last_mod_date" DATETIME NOT NULL  DEFAULT CURRENT_TIMESTAMP ,
    "note" TEXT NOT NULL  DEFAULT "Binary data" ,
    "type" INT NOT NULL DEFAULT 0,
    "size" INTEGER NOT NULL DEFAULT 0,
    "data" BLOB,
    "is_deleted" BOOL NOT NULL  DEFAULT "false",
    FOREIGN KEY("todo_id") REFERENCES "TODO_PROJECT"("todo_id") ON DELETE CASCADE ON UPDATE CASCADE
    );--##

-----SELF TABLE TRIGGERS
--Mother rows
INSERT INTO "FIELD_TYPE" VALUES("0","2011-01-01 00:00:00.000","<action>","false");--##
INSERT INTO "FIELD_TYPE" VALUES("1","2011-01-01 00:00:00.000","<who>","false");--##
INSERT INTO "FIELD_TYPE" VALUES("2","2011-01-01 00:00:00.000","<when>","false");--##
INSERT INTO "FIELD_TYPE" VALUES("3","2011-01-01 00:00:00.000","<where>","false");--##

--PRAGMA FOR RECURSIVE TRIGGERS - !NOTE! IT"S SUPPORTED ONLY FROM SQLITE3 3.6.18+
--PRAGMA recursive_triggers=1;

------FOREIGN KEY TRIGGERS
--ALARM REFERENCES TODO_PROJECT
CREATE TRIGGER _fk_ALARM_delete_0 
    BEFORE DELETE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        DELETE FROM "ALARM"
        WHERE "todo_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_ALARM_insert_0 
    BEFORE INSERT ON "ALARM" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table ALARM violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."todo_id") IS NULL;
    END;--##
CREATE TRIGGER _fk_ALARM_updateParent_0 
    AFTER UPDATE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        UPDATE "ALARM" SET "todo_id" = NEW."todo_id"
        WHERE "todo_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_ALARM_update_0 
    BEFORE UPDATE ON "ALARM" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table ALARM violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."todo_id") IS NULL;
    END;--##

--BINARY REFERENCES TODO_PROJECT
CREATE TRIGGER _fk_BINARY_delete_0 
    BEFORE DELETE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        DELETE FROM "BINARY"
        WHERE "todo_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_BINARY_insert_0 
    BEFORE INSERT ON "BINARY" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table BINARY violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."todo_id") IS NULL;
    END;--##
CREATE TRIGGER _fk_BINARY_updateParent_0 
    AFTER UPDATE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        UPDATE "BINARY" SET "todo_id" = NEW."todo_id"
        WHERE "todo_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_BINARY_update_0 
    BEFORE UPDATE ON "BINARY" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table BINARY violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."todo_id") IS NULL;
    END;--##

--FIELD REFERENCES TODO_PROJECT
CREATE TRIGGER _fk_FIELD_delete_0
    BEFORE DELETE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        DELETE FROM "FIELD"
        WHERE "todo_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_FIELD_insert_0 
    BEFORE INSERT ON "FIELD" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table FIELD violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."todo_id") IS NULL;
    END;--##
CREATE TRIGGER _fk_FIELD_updateParent_0 
    AFTER UPDATE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        UPDATE "FIELD" SET "todo_id" = NEW."todo_id"
        WHERE "todo_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_FIELD_update_0 
    BEFORE UPDATE ON "FIELD" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table FIELD violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."todo_id") IS NULL;
    END;--##

--RECURRENCE REFERENCES TODO_PROJECT
CREATE TRIGGER _fk_RECURRENCE_delete_0 
    BEFORE DELETE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        DELETE FROM "RECURRENCE"
        WHERE "sample_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_RECURRENCE_insert_0 
    BEFORE INSERT ON "RECURRENCE" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table RECURRENCE violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."sample_id") IS NULL;
    END;--##
CREATE TRIGGER _fk_RECURRENCE_updateParent_0 
    AFTER UPDATE ON "TODO_PROJECT" FOR EACH ROW BEGIN
        UPDATE "RECURRENCE" SET "sample_id" = NEW."todo_id"
        WHERE "sample_id" = OLD."todo_id";
    END;--##
CREATE TRIGGER _fk_RECURRENCE_update_0 
    BEFORE UPDATE ON "RECURRENCE" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table RECURRENCE violates foreign key constraint")
        WHERE  (SELECT "todo_id" FROM "TODO_PROJECT" WHERE "todo_id" = NEW."sample_id") IS NULL;
    END;--##

--TODO_PROJECT REFERENCES RECURRENCE
CREATE TRIGGER _fk_RECURRENCE_delete_1 
    BEFORE DELETE ON "RECURRENCE" FOR EACH ROW BEGIN
        DELETE FROM "TODO_PROJECT"
        WHERE "recurrence_id" = OLD."recurrence_id";
    END;--##

--FIELD REFERENCES FIELD_TYPE
CREATE TRIGGER _fk_FIELD_delete_1 
    BEFORE DELETE ON "FIELD_TYPE" FOR EACH ROW BEGIN
        DELETE FROM "FIELD"
        WHERE "field_type_id" = OLD."field_type_id";
    END;--##
CREATE TRIGGER _fk_FIELD_insert_1 
    BEFORE INSERT ON "FIELD" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table FIELD violates foreign key constraint")
        WHERE  (SELECT "field_type_id" FROM "FIELD_TYPE" WHERE "field_type_id" = NEW."field_type_id") IS NULL;
    END;--##
CREATE TRIGGER _fk_FIELD_updateParent_1 
    AFTER UPDATE ON "FIELD_TYPE" FOR EACH ROW BEGIN
        UPDATE "FIELD" SET "field_type_id" = NEW."field_type_id"
        WHERE "field_type_id" = OLD."field_type_id";
    END;--##
CREATE TRIGGER _fk_FIELD_update_1 
    BEFORE UPDATE ON "FIELD" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table FIELD violates foreign key constraint")
        WHERE  (SELECT "field_type_id" FROM "FIELD_TYPE" WHERE "field_type_id" = NEW."field_type_id") IS NULL;
    END;--##

--CHECK TYPE CONSTRAINTS
CREATE TRIGGER _type_BINARY_insert
    BEFORE INSERT ON "BINARY" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table BINARY violates type constraint")
        WHERE TYPEOF(new."size") != 'integer' OR new."size" <0;
    END;--##
CREATE TRIGGER _type_BINARY_update
    BEFORE UPDATE ON "BINARY" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table BINARY violates type constraint")
        WHERE TYPEOF(new."size") != 'integer' OR new."size" <0;
    END;--##
CREATE TRIGGER _type_RECURRENCE_insert
    BEFORE INSERT ON "RECURRENCE" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table RECURRENCE violates type constraint")
        WHERE new."range_start" > new."range_end";
    END;--##
CREATE TRIGGER _type_RECURRENCE_update
    BEFORE INSERT ON "RECURRENCE" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table RECURRENCE violates type constraint")
        WHERE new."range_start" > new."range_end";
    END;--##
CREATE TRIGGER _type_TODO_PROJECT_insert
    BEFORE INSERT ON "TODO_PROJECT" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "insert on table TODO_PROJECT violates type constraint")
        WHERE new."start_date" > new."end_date";
    END;--##
CREATE TRIGGER _type_TODO_PROJECT_update
    BEFORE INSERT ON "TODO_PROJECT" FOR EACH ROW BEGIN
        SELECT RAISE(ROLLBACK, "update on table TODO_PROJECT violates type constraint")
        WHERE new."start_date" > new."end_date";
    END;--##

CREATE INDEX "alarm_by_todo_id" ON "ALARM" ("todo_id" ASC);--##
CREATE INDEX "field_by_todo_id" ON "FIELD" ("todo_id" ASC);--##
CREATE INDEX "binary_by_todo_id" ON "BINARY" ("todo_id" ASC);--##
CREATE INDEX "recurrence_by_sample_id" ON "RECURRENCE" ("sample_id" ASC);--##
